/********************************************************
 * programme TCO pour le passage a niveau
 * communique via le bus CAN
 **************************************************************/
 
#include "CANbusInterface.h"   // interface simplifiant l'utilisation du bus CAN

// option pour la console
#define CONSOLE                     // a supprimer pour une utilisation autonome 
#define DEBIT_CONSOLE 9600          // debit pour la console      

/******************************************************** 
*  parametres du bus CAN
*********************************************************/
 
#define CS 10                     // la broche CS 10 pour NANO/MINI/UNO, 53 pour MEGA
#define DEBIT 500                 // debit du bus CAN
#define ID_MODE 0                 // mode 0 : normal, 1 : etendu
#define ID_DISTANT 0x01          // ID du passage a niveau
#define ID_LOCAL 0x02            // ID du TCO
#define MASK 0xFFF               // masque integral, on ne prend que l'ID du passage a niveau

// les codes des commandes
#define RESET 0x01              // code emission du du reset
#define TRIAGE_ON 0x02          // code emission triage ON
#define TRIAGE_OFF 0x03         // code emission triage OFF
#define RESET_CONF 0x04         // code reception confirmation reset
#define TRIAGE_ON_CONF 0x05     // code reception confirmation triage ON
#define TRIAGE_OFF_CONF 0x06    // code reception confirmation triage OFF
#define LED_ON 0x07             // code reception led ON
#define LED_OFF 0x08            // code reception led OFF

// une instance du bus CAN
CANbusInterface can(CS,DEBIT,ID_MODE);

// les messages pour le bus CAN
messageCAN messageR;
messageCAN messageS;

/********************************************************
* les composants du TCO
* *********************************************************/

// bouton pour le mode triage
int pinCdeTriage = 3;
bool triage = false;

// bouton pour le reset
int pinCdeReset = 4;
bool reset =  false;

// led temoin de l'etat du passage a niveau
int pinLed = 5;

// led temoin du mode triage
int pinTriage = 6;

/********************************************************
 * les methodes du bus CAN
 **********************************************************/

// la routine d'interruption du bus CAN

volatile bool FlagReceive = false;  // drapeau d'IT du bus CAN
void MCP2515_ISR() {FlagReceive = true;} // mise a jour du drapeau d'IT sur arrivee d'un message

// initialisation du bus CAN

void initCAN () {
  while (true) {
    if (can.init(MASK,ID_LOCAL)) {
      attachInterrupt(digitalPinToInterrupt(2),MCP2515_ISR,FALLING);        // attachement de la routine d'IT sur la pin 2 (IT0)
        #ifdef CONSOLE
          Serial.println(F("Initialisation du bus CAN reussie"));
          Serial.println();
        #endif
      break;}
    else {
      #ifdef CONSOLE
        Serial.println(F("initialisation du bus CAN impossible"));
      #endif
      }
  }
}

// gestion des codes recus depuis le passage a niveau

void processMessage() 
          {
      switch (messageR.buf[0]) {
        case RESET_CONF : {digitalWrite(pinTriage,HIGH);digitalWrite(pinLed,HIGH);      // confirmation de reset
                          #ifdef CONSOLE
                          Serial.println(F("TCO <== PAN : CONFIRMATION RESET"));
                          #endif
                          break;}
        case TRIAGE_ON_CONF : {digitalWrite(pinTriage,LOW);                           // confirmation du mode triage
                          #ifdef CONSOLE
                          Serial.println(F("TCO <== PAN : CONFIRMATION MODE TRIAGE ACTIF"));
                          #endif
                          break;}
        case TRIAGE_OFF_CONF : {digitalWrite(pinTriage,HIGH);                         // confirmation arret du mode normal
                        #ifdef CONSOLE
                        Serial.println(F("TCO <== PAN : CONFIRMATION MODE TRIAGE INACTIF"));
                        #endif
                        break;}
        case LED_ON : {digitalWrite(pinLed,LOW);                                    // led temoin allumee
                      #ifdef CONSOLE
                      Serial.println(F("TCO <== PAN : BARRIERES FERMEES"));
                      #endif
                      break;}
        case LED_OFF : {digitalWrite(pinLed,HIGH);                                  // led temoin eteinte
                      #ifdef CONSOLE
                      Serial.println(F("TCO <== PAN : BARRIERES OUVERTES"));
                      #endif
                      break;}
      }
}

// envoie des messages CAN au passage a niveau

void sendMessage() {
// test si envoie d'une demande de mode triage
  if ((digitalRead(pinCdeTriage) == LOW) && (!triage)) {messageS.buf[0] = TRIAGE_ON; can.writeMessage(messageS); 
                                                         #ifdef CONSOLE
                                                         Serial.println(F("TCO ==> PAN :  MODE TRIAGE ACTIF")); 
                                                         #endif
                                                         triage = true;}
                                                         
  // test si envoie de l'arret du mode triage
  if ((digitalRead(pinCdeTriage) == HIGH) && (triage)) {messageS.buf[0] = TRIAGE_OFF; can.writeMessage(messageS);
                                                        #ifdef CONSOLE
                                                        Serial.println(F("TCO ==> PAN :  MODE TRIAGE INACTIF"));
                                                        #endif
                                                        triage = false;}
                                                                                                               
   // test si envoie d'un reset                             
  if ((digitalRead(pinCdeReset) == LOW)&& (!reset)) {messageS.buf[0] = RESET; can.writeMessage(messageS);
                                                        digitalWrite(pinTriage,HIGH); 
                                                        digitalWrite(pinLed,HIGH);
                                                        #ifdef CONSOLE
                                                        Serial.println(F("TCO ==> PAN : RESET")); 
                                                        #endif
                                                        reset = true;}
                                                        
   // stop le reset
  if (digitalRead(pinCdeReset) == HIGH) {reset = false;}
}

/**************************************************
 * affichage de la configuration
 *************************************************/

void affiche() {
  Serial.println(F("Configuration du TCO du passage a niveau"));
  Serial.println();
  Serial.print(F("switch du mode triage sur pin : pin")); Serial.println(pinCdeTriage);
  Serial.print(F("bouton reset sur pin : pin")); Serial.println(pinCdeReset);
  Serial.print(F("Led temoin passage ferme sur pin : pin")); Serial.println(pinLed);
  Serial.print(F("Led temoin mode triage ON sur pin : pin")); Serial.println(pinTriage);
  Serial.println();
 }
 
// le setup

void setup() {

  /********************************************
   * pour la console
   ***********************************************/
   
  #ifdef CONSOLE
    Serial.begin(DEBIT_CONSOLE);
    affiche();
  #endif

  /*************************************************
   * initialisation des broches
   ******************************************************/
   
  pinMode(pinCdeTriage,INPUT_PULLUP);
  pinMode(pinCdeReset,INPUT_PULLUP);
  pinMode(pinLed,OUTPUT); digitalWrite(pinLed,HIGH);
  pinMode(pinTriage,OUTPUT); digitalWrite(pinTriage,HIGH);

  /*******************************************************
   * initialisation du bus CAN
   *******************************************************/
   
  initCAN();
  
  //preformattage des messages CAN 
  
  messageS.id = ID_DISTANT;       // un seul ID, le passage a niveau
  messageS.len = 1;               // une donnee sur un octet, le code
}

// la boucle

void loop() {

  /***********************************************************
   * reception des messages du bus CAN
   ************************************************************/
   
  if (FlagReceive) {FlagReceive = false; can.receiveMessage();}   // ssi message recu du bus les mettre dans le buffer circulaire

  /***************************************************************
   * lecture des messages arrivees en attente
   *****************************************************************/
   
  if (can.readMessage(messageR)){processMessage();}               // si message en attente de traitement
  
/********************************************************************
 * envoie des commandes au passage à niveau
 ******************************************************************/
 sendMessage();
}
